#uses "fwSystemOverview/fwSystemOverview.ctl"

const string modFsmNodeProperties = "modFsmNodeProperties";

//fwTree_getAllTreeNodes: 
/**  Get all nodes belonging to a Tree.
 	
Usage: JCOP framework internal, public

PVSS manager usage: VISION, CTRL

      @param tree: The name of the tree
      @param nodes: The names of all nodes
      @param exInfo: exception return
*/
 
fwSysOverviewFsm_getAllTreeNodes(string tree, dyn_string &nodes, dyn_string &exInfo)
{
  dyn_string children;
  int i;
  string sys = "";
  if(strpos(tree, ":"))
  {
    sys = dpSubStr(tree, DPSUB_SYS);
  }
  
 fwTree_getChildren(tree, children, exInfo);
  int n = dynlen(children);
	 for(i = 1; i <= n; i++)
	 {
    //resolve the system of the child
    if(strpos(children[i], ":") < 0)
    {
      if(strpos(children[i], "&") >= 0)
      {
        dyn_string ex;
        string device = fwTree_getNodeDisplayName(children[i], ex);
        children[i] = device;
      }
      
      string childDp = "";
      fwCU_getDp(children[i], childDp);
      string childSys = dpSubStr(childDp, DPSUB_SYS);
      children[i] = childSys + children[i];
    }    
    
//     if(strpos(children[i], "&") >= 0)
//     {
      //the node is a reference, get the real node:
//       dyn_string ex;
//       string type = "";
//       string device = "";
//       
//       fwTree_getNodeDevice(children[i], device, type, ex);
     // DebugN("*************children[i], device", children[i], device);
//     }
    

    
		  dynAppend(nodes, children[i]);
		  fwSysOverviewFsm_getAllTreeNodes(children[i], nodes, exInfo);
	 }
  
  return;  
}

string fwSysOverviewFsm_getProjectState(string projectDp)
{
  string fsmState;
  string projectDetailedState = fwSysOverview_getProjectState(projectDp);
  
  if(projectDetailedState == "RUNNING" || projectDetailedState == "STOPPED")
    fsmState = projectDetailedState;
  else
    fsmState = "ERROR";
  
  return fsmState;
}

dyn_string fwSysOverviewFsm_getTreeDevices(string tree, bool uniqueDeviceList = false, string pattern = "")
{
  dyn_string devices = fwUi_getIncludedTreeDevices(tree);
  if(uniqueDeviceList)
    dynUnique(devices);
  
  if(pattern != "")
  {
    dyn_string tmp;
    for(int i=1; i<=dynlen(devices); i++)
    {
      strreplace(devices[i], getSystemName(), "");
      if (patternMatch(pattern, devices[i]))
      {
        dynAppend(tmp, devices[i]);
      }
    }
    devices = tmp;
  }
  
  return devices;
}

string fwSysOverviewFsm_getTreeByLabel(string treeLabel)
{
  string tree;
  dyn_string soTrees;
  fwSystemOverviewFsm_getTrees(soTrees);
  for(int i=1; i<=dynlen(soTrees); i++)
  {
    string label;
    fwUi_getLabel(soTrees[i], soTrees[i], label);
    if (label == treeLabel)
    {
      tree = soTrees[i];
      break;
    }
  }
  return tree;
}

dyn_string fwSysOverviewFsm_getChildrenOfType(string tree, string childrenDpType)
{
  //Resolve the system name if the tree does not contain it:
  string sys = "";
  if(strpos(tree, ":") < 0)
  {
    //find out the system the node is defined in  
    string nodeDp = "";
    fwCU_getDp(tree, nodeDp);
    sys = dpSubStr(nodeDp, DPSUB_SYS); 
    tree = sys + tree;
  }
  
  dyn_string ex, nodes;
//  fwSysOverviewFsm_getAllTreeNodes(tree, nodes, ex);
 
  int pos = strpos(tree, ":");
  if(pos >= 0)
  {
    tree=substr(tree, pos + 1, strlen(tree));
  }
  fwUi_getTreeDevices(tree, nodes); 
  
  int n = dynlen(nodes);
  dyn_string children;
  string child;
  for(int i = 1; i <= n; i++)
  {
    if(patternMatch("*_FWDM", nodes[i])){
      continue;
    }
      
    if(strpos(nodes[i], ":") < 0)
    {
      nodes[i] = sys + nodes[i];
    }

    
    child = nodes[i];
    string dp = "";
    
    if(dpExists(nodes[i]))
    {
      dp = nodes[i];
    }
    else if (patternMatch("*:&*", nodes[i]) > 0)
    {
      child = fwTree_getNodeDisplayName(nodes[i], ex);
      if(strpos(child, ":") < 0)
      {
        child = sys + child;
      }
      
      if (!dpExists(child))
      {
        dp = dpAliasToName(child);
      }
      else
      {        
        dp = child;
      }
    }
    else
    {
      dp = dpAliasToName(nodes[i]);
    }
    


    if(dpExists(dp) && dpTypeName(dp) == childrenDpType)
    {
      if (!dynContains(children, child))
      {        
        dynAppend(children, child);
      }
    }
  }
  //DebugN("****childrenType, devices", childrenDpType, children);
  return children;  
} 

dyn_dyn_string fwSysOverviewFsm_getChildrenOfTypeWithDomain(string tree, string childrenOfType, string controlUnit)
{
  dyn_dyn_string children;
  _fwSysOverviewFsm_getChildrenOfTypeWithDomain(tree, childrenOfType, controlUnit, children);
  return children;
}
void _fwSysOverviewFsm_getChildrenOfTypeWithDomain(string tree, string childrenOfType, string controlUnit, dyn_dyn_string& children)
{
  //Resolve the system name if the tree does not contain it:
  string sys = "";
  if(strpos(tree, ":") < 0)
  {
    //find out the system the node is defined in  
    string nodeDp = "";
    fwCU_getDp(tree, nodeDp);
    sys = dpSubStr(nodeDp, DPSUB_SYS); 
    tree = sys + tree;
  }
  
  dyn_string ex, nodes;
  dynClear(ex);
  fwSysOverviewFsm_getAllTreeNodes(tree, nodes, ex);
  
  int n = dynlen(nodes);
  string child;
  int childrenLength = dynlen(children);
  for(int i = 1; i <= n; i++)
  {
    if(strpos(nodes[i], ":") < 0)
    {
      nodes[i] = sys + nodes[i];
    }

    
    child = nodes[i];
    string dp = "";
    
    if(dpExists(nodes[i]))
    {
      dp = nodes[i];
    }
    else if (patternMatch("*:&*", nodes[i]) > 0)
    {
      child = fwTree_getNodeDisplayName(nodes[i], ex);
      if(strpos(child, ":") < 0)
      {
        child = sys + child;
      }
      
      if (!dpExists(child))
      {
        dp = dpAliasToName(child);
      }
      else
      {        
        dp = child;
      }
    }
    else
    {
      dp = nodes[i]; //in reality this is not a dp but an alias
    }

//////
    if (_fwSysOverviewFsm_isOfType(dp, childrenOfType))
    {    
      if (!_fwSysOverviewFsm_dynContains(children, dp))
      {
          ++childrenLength;
          children[childrenLength][1] = dp;
          children[childrenLength][2] = controlUnit;
      }
//      foundChildrenCount++;
    }
      
    if (fwSysOverviewFsm_isDomain(nodes[i]))
      _fwSysOverviewFsm_getChildrenOfTypeWithDomain(child, childrenOfType, child, children);
    else
      _fwSysOverviewFsm_getChildrenOfTypeWithDomain(child, childrenOfType, controlUnit, children);
    
//////    
  } 
  
  
/* 
  
  //Resolve the system name if the tree does not contain it:
  string sys = "";
  if(strpos(tree, ":") < 0)
  {
    //find out the system the node is defined in  
    string nodeDp = "";
    fwCU_getDp(tree, nodeDp);
    sys = dpSubStr(nodeDp, DPSUB_SYS); 
    tree = sys + tree;
  }
  
  dyn_string ex, nodes;
  string node;
//fwInstallation_throw(1);  

//   if(strpos(tree, "&") >= 0)
//   {
//     string device, type;
//     dyn_string ex;
//     
//     fwTree_getNodeDevice(tree, device, type, ex);
//     tree = device;
//   }
  
  
  fwTree_getChildren(tree, nodes, ex);
//  dyn_int nodeTypes;  
//  nodes = fwCU_getChildren(nodeTypes, tree);
  
//fwInstallation_throw(2);  
  int childrenLength = dynlen(children);
//  int foundChildrenCount = 1;
  int n = dynlen(nodes);
  for(int i=1; i<= n; i++)
  {
    if(strpos(nodes[i], ":") < 0)
    {
      nodes[i] = sys + nodes[i];
    }
    
    if(strpos(nodes[i], "&") >= 0) //if the child is a reference, point to the real device
    {
      string device = "";
      string type = "";
      dyn_string ex;
      fwTree_getNodeDevice(nodes[i], device, type, ex);
      nodes[i] = device;
      if(strpos(nodes[i], ":") < 0)
      {
        nodes[i] = sys + nodes[i];
      }
    }

    if (_fwSysOverviewFsm_isOfType(nodes[i], childrenOfType))
    {
      if (strpos(nodes[i], "&") >= 0) 
      {
        node = fwTree_getNodeDisplayName(nodes[i], ex) ;
        if(strpos(node, ":") < 0) //necessary as getnodedsiplayname removes the system name
        {
          node = sys + node;
        }
        
if(node == "")
{
 DebugN("111111111 Inserting emtpy node now: sys, node, nodes[i], tree", sys, node, nodes[i], tree);  
}


      if (node != "" && !_fwSysOverviewFsm_dynContains(children, node))
        {
          ++childrenLength;
          children[childrenLength][1] = node;
          children[childrenLength][2] = controlUnit;
        }
        
      }
      else if (!_fwSysOverviewFsm_dynContains(children, nodes[i]))
      {
if(nodes[i] == "")
{
 DebugN("2222222222 Inserting emtpy node now: sys, node, nodes[i], tree", sys, node, nodes[i], tree);  
}
        ++childrenLength;
        children[childrenLength][1] = nodes[i];
        children[childrenLength][2] = controlUnit;
      }
//      foundChildrenCount++;
    }
    
    if (fwSysOverviewFsm_isDomain(nodes[i]))
      _fwSysOverviewFsm_getChildrenOfTypeWithDomain(nodes[i], childrenOfType, nodes[i], children);
    else
      _fwSysOverviewFsm_getChildrenOfTypeWithDomain(nodes[i], childrenOfType, controlUnit, children);
  }
*/  
//fwInstallation_throw(3);  
}

/** This funciton is introduce to work around the performance issues of fwFsm_isDomain 
which internally uses a mapping*/
bool fwSysOverviewFsm_isDomain(string domain)
{
  string sys = "";
  int pos = strpos(domain, ":");
  if(pos >= 0)
  {
    sys = substr(domain, 0, pos + 1);
    domain = substr(domain, pos + 1);
  }
  
  return dpExists(sys + "fwCU_"+domain);  
}

string fwSysOverviewFsm_getDomain(string device)
{
  string domain;
  dyn_string devices = dpNames("*|" + device, "_FwFsmDevice");
  if (dynlen(devices) >0)
  {
    dyn_string tmpArr = strsplit(dpSubStr(devices[1], DPSUB_DP), "|");
    domain = tmpArr[1];  
  }
  return domain;
}

bool _fwSysOverviewFsm_dynContains(dyn_dyn_string ddsArray, string element)
{
  for(int i=1;i <= dynlen(ddsArray); i++)
  {
    for (int j=1; j<= dynlen(ddsArray[i]); j++)
    {
      if (ddsArray[i][j] == element)
        return true;    
    }    
  }
  
  return false;
}

bool _fwSysOverviewFsm_isOfType(string node, string type)
{
  bool isOfType = false;
  dyn_string ex; 
  string dp = "";
  string child;
  
  if(dpExists(node))
  {
    dp = node;
  }
  else if (patternMatch("*:&*", node) > 0)
  {
    child = fwTree_getNodeDisplayName(node, ex);
    if(strpos(child, ":") < 0)
    {
      string childDp;
      fwCU_getDp(child, childDp);
      string sys = dpSubStr(childDp, DPSUB_SYS);
      
      child = sys + child;
    }
      
    if (!dpExists(child))
    {
      dp = dpAliasToName(child);
    }
    else
    {        
      dp = child;
    }
  }
  else
  {
    dp = dpAliasToName(node);
  }
  
/*  else if (strpos(node, "&") == 0)
  {
    string displayName = fwTree_getNodeDisplayName(node, ex) ;
    if (!dpExists(displayName))
      dp = dpAliasToName(displayName);
    else
      dp = displayName;
  }
  else
    dp = dpAliasToName(node);
*/ 
  
  isOfType = (dpExists(dp) && (dpTypeName(dp) == type));
  return isOfType;
}



/*
string fwFsmUser_nodeEnabled(string parent, string dev)
{  
  fwSysOverview_enableProjectMonitoring(dev);  
  fwSysOverview_stopMonitoring();
  fwSysOverview_startMonitoring();
}

string fwFsmUser_nodeDisabled(string parent, string dev)
{
  fwSysOverview_disableProjectMonitoring(dev);
  fwSysOverview_stopMonitoring();
  fwSysOverview_startMonitoring(); 
}
*/

bool fwSysoverviewFsm_isGranted(string domain)
{
  fwSysOverview_getACDomain(domain);
  dyn_string ex;
  bool isGranted = false; 
  
  if(domain == "")
    isGranted = true;
  else
  {
    string expertPrivilege = fwSysOverview_getExpertPrivilege();
    // check the current  privilege level 
    if (expertPrivilege == "")
    {
      DebugN("No expert privilege defined!");
      return true;
    }
    fwAccessControl_isGranted(domain + ":" +expertPrivilege, isGranted, ex);
  }
  
  return isGranted;
}

int fwSysOverviewFsm_logUserAction(string user, string action, string node)    
{
  dyn_string actions = fwSysOverviewFsm_getUserActions();
  string t = getCurrentTime();
  dynAppend(actions, user + " - " + action + " - " + node + " - " + t);
  return fwSysOverviewFsm_setUserActions(actions);
}

dyn_string fwSysOverviewFsm_getUserActions()    
{
  dyn_string actions;
  dpGet("fwSystemOverview_fsmUserActions.", actions);
  
  return actions;
}
int fwSysOverviewFsm_setUserActions(dyn_string actions)    
{
  return dpSet("fwSystemOverview_fsmUserActions.", actions);
}
    
    
fwFsmUser_operateNodeRightClick(string domain, string obj)
{
  //Check if the user is allowed:
  bool isGranted = fwSysoverviewFsm_isGranted(domain);
  //check if the tree is included or excluded:
  bool isIncluded = false;
  string mode = "";
  string type = "";
  string parentDomain = "";
  int isCu = fwFsm_isCU(domain, obj);


   if(isCu)
   {
    mode = fwFsmUi_getCUMode(domain, obj);
    isIncluded = strtolower(mode)=="included"?TRUE:FALSE;
   }
   else
   {
     mode = fwUi_getDUMode(domain, obj);
    isIncluded = strtolower(mode)=="enabled"?TRUE:FALSE;
   }
 
  string action = "UNSET_INTERVENTION_MODE";
  if(isIncluded)
  {
    action = "SET_INTERVENTION_MODE";
  }
  
  dyn_string menu = makeDynString("PUSH_BUTTON, " + action + ", 1, " + isGranted);
  int ret = 0;
  popupMenu(menu, ret);
  if(ret == 1)
  {
    dyn_string ds;
    dyn_float df;
    string user = getUserName();
    
    ChildPanelOnCentralModalReturn("vision/MessageWarning2", "Enable/disable apps", makeDynString("$1:" + user + ", are you sure you want to\n" + action + " for " + obj + "?\nThis action will be logged.", "$2:Yes", "$3:Cancel"), df, ds);    
    if(dynlen(df) <= 0 || df[1] < 1)
    {
      //user pressed cancel
      return;
    }

    fwSysOverviewFsm_logUserAction(user, action, obj);    
    
    //get the current owner of the subtree and supplant his/her personality
    string id = "";
    if(isIncluded)
    {
      if(isCu)
      {
        fwUi_getOwnership(obj, id);
        type = "";
        parentDomain = fwCU_getParent(type, domain);
        fwCU_excludeTree(parentDomain, obj, id, 1);
      }
      else
      {
        fwCU_disableObj(domain, obj);
      }
    }
    else
    {
      type = "";
      parentDomain = fwCU_getParent(type, domain);
      if(isCu)
      {
        fwUi_getOwnership(obj, id);
        
        fwCU_includeTree(parentDomain, obj, id, 1);
      }
      else
      {
        fwCU_enableObj(domain, obj);        
      }
    }    
  }
  return;  
}

dyn_string fwSysOverviewFsm_getApplicationNodes(string application)
{
  dyn_string hostDps = fwSysOverviewFsm_getChildrenOfType(application, "FwFMCNode");
  dyn_string hosts;
  
  for(int i = 1; i <= dynlen(hostDps); i++)
    dynAppend(hosts, fwFMC_getNodeName(hostDps[i]));

  return hosts;  
}

fwFsmUser_operateNodeClick(string domain, string obj)
{
   string panel = "";
   string panelRef = "";
   dyn_string hosts;
   
  //get rid of the previous panel shown:   
  string oldPanel = tfCurrentFsmNodePanel.text; 
                       
  if(oldPanel != "" && isPanelOpen(oldPanel, modFsmNodeProperties))
    PanelOffModule(oldPanel, modFsmNodeProperties);
  
//  bool state = false;
//  getValue("FW_SYSTEM_OVERVIEW_TOOL.cbEditor", state, 0, state);
  if(cbEditor.state(0))
//  if(state)
  {
    if(fwFsm_isDU(domain, obj))
    {
      string type = dpTypeName(obj);
      
      if(type == "FwFMCNode")
      {
        panel = "fwConfigurationDBSystemInformation/fwConfigurationDBSystemInformation_computerProperties.pnl";
        string node = fwFMC_getNodeName(obj);
        RootPanelOnModule(panel, obj, modFsmNodeProperties, makeDynString("$computer:"+node, "$embedded:TRUE"));
      }
      else if(type == "FwSystemOverviewProject")
      {
        panel = "fwConfigurationDBSystemInformation/fwConfigurationDBSystemInformation_projectProperties.pnl";
        dyn_string ex;
        string project = "";
        fwSysOverview_getProjectDb(obj, project, ex);
        string nodeDp, node;
        fwSysOverview_getProjectPc(obj, nodeDp, ex);
        fwSysOverview_getHostDb(nodeDp, node, ex);

        RootPanelOnModule(panel, obj, modFsmNodeProperties, makeDynString("$computer:"+node, "$project:"+project, "$embedded:TRUE"));
      }      
    }
    else
    {
      hosts = fwSysOverviewFsm_getApplicationNodes(obj);    
      panel = "fwConfigurationDBSystemInformation/fwConfigurationDBSystemInformation_systemManager.pnl";
      RootPanelOnModule(panel, obj, modFsmNodeProperties, makeDynString("$computer:"+hosts));
    }
  }
  else
  {
    fwUi_getUserPanel(domain, obj, panel);
    if(panel != "")
      panel = "fwSystemOverview/Fsm/" + panel;
     
    panelRef = domain + "_specific";
    RootPanelOnModule(panel, obj, modFsmNodeProperties, makeDynString(domain, obj));
  }  
  
  tfCurrentFsmNodePanel.text = panel;
     
  return;  
}
 
fwSystemOverviewFsm_getRootNodes(dyn_string &parents, dyn_string &exInfo)
{
  dyn_string dps;
  int i;
  string node;

	dynClear(parents);
	dps = dpNames("*.","_FwTreeNode");
	for(i = 1; i <= dynlen(dps); i++)
	{
		dps[i] = substr(dps[i],0,strlen(dps[i])-1);
		dpGet(dps[i]+".parent",node);
		if(node == "fwTN_FSM" && !patternMatch("*fwTN_---*", dps[i]))
		{
			dynAppend(parents,_fwTree_getNodeName(dps[i]));
		}
	}
}

int fwSystemOverviewFsm_getTrees(dyn_string &fmcTrees)
{
  dyn_int internalTypes;
  dyn_string trees, ex;
  fwSystemOverviewFsm_getRootNodes(trees, ex);
  for(int i = 1; i <= dynlen(trees); i++)  
    if(patternMatch("fwSO_*", trees[i]))
      dynAppend(fmcTrees, trees[i]);
  
  return 0;
}

bool fwSystemOverviewFsm_isTreeRunning(string tree)
{
  string state;
  fwCU_getState(tree, state);
//  DebugN(state);
  return state == "DEAD"?false:true;
}


